@@ -3,10 +3,10 @@ cat(crayon::yellow("\ntest lmerTest interface and other ANOVA tables:"))
 
 { 
   { # LM (more LMs with sleepstudy data below)
-    fit <- lm(sr ~ ., data = LifeCycleSavings)
+    baselm <- lm(sr ~ ., data = LifeCycleSavings)
     spfit <- fitme(sr ~ pop15+pop75+dpi+ddpi , data = LifeCycleSavings)
     testthat::test_that("whether lm() and fitme ML yield identical anova tables",
-                        testthat::expect_true(diff(range(na.omit(unlist(anova(fit))-unlist(anova(spfit)))))<1e-10))
+                        testthat::expect_true(diff(range(na.omit(unlist(anova(baselm))-unlist(anova(spfit)))))<1e-10))
     spfitRE <- fitme(sr ~ pop15+pop75+dpi+ddpi , data = LifeCycleSavings, method="REML")
     testthat::test_that("whether fitme ML and fitme REML yield identical anova tables for LM",
                         testthat::expect_true(diff(range(na.omit(unlist(anova(spfit))-unlist(anova(spfitRE)))))<1e-10))
@@ -29,24 +29,24 @@ cat(crayon::yellow("\ntest lmerTest interface and other ANOVA tables:"))
       u = c(5,10,15,20,30,40,60,80,100),
       lot1 = c(118,58,42,35,27,25,21,19,18),
       lot2 = c(69,35,26,21,18,16,13,12,12))
-    fit <- glm(lot1 ~ log(u), data = clotting, family = Gamma)
+    baseglm <- glm(lot1 ~ log(u), data = clotting, family = Gamma)
     spglm <- fitme(lot1 ~ log(u), data = clotting, family = Gamma, method="REML")
     # F test returned for Gamma, but large differences are expected from different phi estimation
     #testthat::test_that("whether fitme and poisson glm yield identical anova F tables",
     #                    testthat::expect_true(diff(range(na.omit(unlist(anova(fit, test = "F"))-unlist(anova(spglm, test = "F")))))<1e-3)) 
     testthat::test_that("whether fitme and Gamma glm yield identical anova Cp tables",
-                        testthat::expect_true(diff(range(na.omit(unlist(anova(fit, test = "Cp"))-unlist(anova(spglm, test = "Cp")))))<1e-3)) # some difference -- expected from different phi estimation
+                        testthat::expect_true(diff(range(na.omit(unlist(anova(baseglm, test = "Cp"))-unlist(anova(spglm, test = "Cp")))))<1e-3)) # some difference -- expected from different phi estimation
     # spglm <- fitme(lot1 ~ log(u), data = clotting, family = Gamma, method="ML") # differs more on Cp
     testthat::test_that("whether fitme and Gamma glm yield identical anova Chisq tables",
-                        testthat::expect_true(diff(range(na.omit(unlist(anova(fit, test = "Chisq"))-unlist(anova(spglm, test = "Chisq")))))<1e-10))
+                        testthat::expect_true(diff(range(na.omit(unlist(anova(baseglm, test = "Chisq"))-unlist(anova(spglm, test = "Chisq")))))<1e-10))
     testthat::test_that("whether fitme and Gamma glm yield identical anova Rao [score test] tables",
-                        testthat::expect_true(diff(range(na.omit(unlist(anova(fit, test = "Rao"))-unlist(anova(spglm, test = "Rao")))))<1e-5)) 
+                        testthat::expect_true(diff(range(na.omit(unlist(anova(baseglm, test = "Rao"))-unlist(anova(spglm, test = "Rao")))))<1e-5)) 
     testthat::test_that("whether fitme and Gamma glm yield identical drop1 F tables (apart from AICs)",
-                        testthat::expect_true(diff(range(na.omit(unlist(drop1(fit, test = "F"))-unlist(drop1(spglm, test = "F")))[-(4:5)]))<1e-3)) 
+                        testthat::expect_true(diff(range(na.omit(unlist(drop1(baseglm, test = "F"))-unlist(drop1(spglm, test = "F")))[-(4:5)]))<1e-3)) 
     testthat::test_that("whether fitme and Gamma glm yield identical drop1 Chisq tables (apart from AICs and scaled dev)",
-                        testthat::expect_true(diff(range(na.omit(unlist(drop1(fit, test = "Chisq"))-unlist(drop1(spglm, test = "Chisq")))[-(4:6)]))<1e-3)) 
+                        testthat::expect_true(diff(range(na.omit(unlist(drop1(baseglm, test = "Chisq"))-unlist(drop1(spglm, test = "Chisq")))[-(4:6)]))<1e-3)) 
     testthat::test_that("whether fitme and Gamma glm yield identical drop1 Rao [score test] tables (apart from AICs and and scaled dev)",
-                        testthat::expect_true(diff(range(na.omit(unlist(drop1(fit, test = "Rao"))-unlist(drop1(spglm, test = "Rao")))[-(4:6)]))<1e-3)) # some fifference -- expected from different phi estimation
+                        testthat::expect_true(diff(range(na.omit(unlist(drop1(baseglm, test = "Rao"))-unlist(drop1(spglm, test = "Rao")))[-(4:6)]))<1e-3)) # some fifference -- expected from different phi estimation
   }
 
   # LM and LMM
