@@ -1,4 +1,4 @@
-cat(crayon::yellow("\ntest poly():\n"))
+cat(crayon::yellow("\ntest poly():\n")) 
 
 set.seed(123)
 d <- data.frame(x = 1:10, y = rnorm(10), z=rnorm(10))
@@ -17,6 +17,34 @@ m4 <- fitme(y ~ poly(cbind(x,z), 2, raw = TRUE), data = d)
 m5 <- fitme(y ~ poly(cbind(x,z), 2), data = d)
 testthat::expect_true(diff(range(c(-7.60735012544,logLik(m4),logLik(m5))))<1e-7) 
 
+{
+  library("splines")
+  # requireNamespace() 'loads' the namespace
+  # library() or require() additionally attach it to the search list:
+  # the latter is required for use in a formula.
+  
+  mbs <- fitme(y ~ bs(x), data = d) 
+  subid <- c(4,3,2)
+  p1 <- predict(mbs)[subid] 
+  p2 <- predict(mbs, newdata = mbs$data)[subid]
+  p3 <- predict(mbs, newdata = mbs$data[subid,])[,1] 
+  # knots=NULL 'results in a basis for ordinary polynomial regression'
+  m4 <- fitme(y ~ poly(x, 3L), data = d)
+  p4 <- predict(m4, newdata = mbs$data[subid,])[,1] # =>
+  # if I use here the bigger data used in modtp <- fitme() LMM below,
+  # then the above predict generates a warning while the comparable code using lm() doesn't
+  # But the difference seems due only to floating point inaccuracy 
+  # in a comparison of 'x' values to the bounds of their range, in splines::bs()
+  testthat::expect_true(max(abs(range(c(p1-p2,p1-p3,p1-p4))))<1e-7) 
+  
+  mns <- fitme(y ~ ns(x, df=2), data = d) 
+  p1 <- predict(mns)[subid] 
+  p2 <- predict(mns, newdata = mns$data)[subid]
+  p3 <- predict(mns, newdata = mns$data[subid,])[,1] 
+  testthat::expect_true(max(abs(range(c(p1-p2,p1-p3))))<1e-7) 
+  
+  detach("package:splines")
+}
 
 ## with a random effect
 set.seed(123)
@@ -61,3 +89,4 @@ if(requireNamespace("lme4", quietly = TRUE)) {
   testthat::test_that(paste0("criterion for ratios of predictions was ",signif(crit,6)," from 1"), # affected by xtol_abs_factors$rcLam
                       testthat::expect_true(crit<3e-5)) # not precise, but the variance estimates are large
 }
+
