@@ -12,14 +12,15 @@ if (file.exists((privdata <- paste0(spaMM::projpath(),"/../tests_misc/ranCoefs/a
     # final precision depends on two steps ! And ultimately on HLfit/hatval precision in the refit
     testval3o <- 1559.81264462379 ## sph gives practically the same result but longer 
     zut <- try(testthat::expect_equal((res3o <- attr(attr(fitme3,"optimInfo")$optim.pars,"optr")$objective),
-                                      testval3o,tol=1e-8),silent=TRUE)
+                                      testval3o,tolerance=1e-8),silent=TRUE)
     if (inherits(zut,"try-error")) {
       if (res3o>testval3o) strg <- " poorer" else strg <- " better" 
       cat(paste0("objective from fitme3's optimInfo=",
                  res3o, strg, " than tested value ",testval3o," by ",res3o-testval3o ,"\n"))
     }
     testval3f <- -1559.812644545275
-    zut <- try(testthat::expect_equal((res3f <- logLik(fitme3)[[1L]]),testval3f,tol=1e-8),silent=TRUE) 
+    zut <- try(testthat::expect_equal((res3f <- logLik(fitme3)[[1L]]),
+                                      testval3f,tolerance=1e-8),silent=TRUE) 
     if (inherits(zut,"try-error")) {
       if (res3f<testval3f) strg <- " poorer" else strg <- " better" 
       cat(paste0("logLik(fitme3)=",
@@ -36,14 +37,15 @@ if (file.exists((privdata <- paste0(spaMM::projpath(),"/../tests_misc/ranCoefs/a
     #
   }
   if (spaMM.getOption("example_maxtime")>20) {
-    if (file.exists((privtest <- paste0(spaMM::projpath(),"/package/tests_private/test-rc_transf.R")))) {
+    if (file.exists((privtest <- paste0(spaMM::projpath(),"/package/tests_private/test-rC_transf.R")))) {
       source(privtest)
     }
     (HLfit3 <- HLfit(total_red ~ sex*env + (1|rep) + (0 + env|line),
                      data = my.data, HLmethod="ML"))
     how(HLfit3) # 2.2s v3.0.34
-    testval3h <- -1559.81264432437 # chol -- better than ever
-    zut <- try(testthat::expect_equal((res3h <- logLik(HLfit3)[[1L]]),testval3h,tol=1e-8),silent=TRUE) 
+    testval3h <- -1559.81264432437 # chol -- better than ever; fit is singular...
+    zut <- try(testthat::expect_equal((res3h <- logLik(HLfit3)[[1L]]),
+                                      testval3h,tolerance=1e-8),silent=TRUE) 
     if (inherits(zut,"try-error")) {
       if (res3h<testval3h) strg <- " poorer" else strg <- " better" 
       cat(paste0("logLik(HLfit3)=",
@@ -64,14 +66,15 @@ if (file.exists((privdata <- paste0(spaMM::projpath(),"/../tests_misc/ranCoefs/a
     }
     testval6o <- 1536.080805076491 # chol -- good 
     zut <- try(testthat::expect_equal((res6o <- attr(attr(fitme6,"optimInfo")$optim.pars,"optr")$objective), 
-                                      testval6o,tol=1e-8),silent=TRUE)
+                                      testval6o,tolerance=1e-8),silent=TRUE)
     if (inherits(zut,"try-error")) {
       if (res6o>testval6o) strg <- " poorer" else strg <- " better" 
       cat(paste0("objective from fitme6's optimInfo=",
                  res6o, strg, " than tested value ",testval6o," by ",res6o-testval6o ,"\n"))
     }
     testval6f <- -1536.080805075333 # chol -- good 
-    zut <- try(testthat::expect_equal((res6f <- logLik(fitme6)[[1L]]),testval6f,tol=1e-8),silent=TRUE) 
+    zut <- try(testthat::expect_equal((res6f <- logLik(fitme6)[[1L]]),
+                                      testval6f,tolerance=1e-8),silent=TRUE) 
     if (inherits(zut,"try-error")) {
       if (res6f<testval6f) strg <- " poorer" else strg <- " better" 
       cat(paste0("logLik(fitme6)=",
@@ -81,7 +84,8 @@ if (file.exists((privdata <- paste0(spaMM::projpath(),"/../tests_misc/ranCoefs/a
                      data = my.data, HLmethod="ML"))
     how(HLfit6) # 12s v3.0.34; quite longer (35.7s) in v3.0.44 which gives more exact result.
     testval6h <- -1536.08079543165 # chol -- good 
-    zut <- try(testthat::expect_equal((res6h <- logLik(HLfit6)[[1L]]),testval6h,tol=1e-8),silent=TRUE) 
+    zut <- try(testthat::expect_equal((res6h <- logLik(HLfit6)[[1L]]),
+                                      testval6h,tolerance=1e-8),silent=TRUE) 
     if (inherits(zut,"try-error")) { 
       if (res6h<testval6h) strg <- " poorer" else strg <- " better" 
       cat(paste0("logLik(HLfit6)=",
